package net.TJ09.Teleporter;

import java.util.logging.Logger;

import java.io.File;
import java.util.HashSet;

import org.bukkit.Server;

import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginLoader;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.configuration.file.FileConfiguration;

import org.bukkit.block.Block;
import org.bukkit.Material;
import org.bukkit.Location;
import org.bukkit.util.Vector;
import org.bukkit.entity.Player;

import org.bukkit.event.Listener;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.block.Action;

import ru.tehkode.permissions.PermissionManager;
import ru.tehkode.permissions.bukkit.PermissionsEx;

/**
 * Teleporter for Bukkit
 *
 * @author TJ09
 */
public class Teleporter extends JavaPlugin implements Listener {
	private static PermissionManager permissions;
	
	public final HashSet<Byte> ignoreBlocks = new HashSet<Byte>();
	public int teleBlock = 0;

	public void onEnable() {
		// Make the data folder for the plugin where configuration files
        // and other data files will be stored
        getDataFolder().mkdirs();
        
        //Load config
		getConfig().addDefault("ignoreBlocks","0,6,8,9,10,11,27,28,37,38,39,40,50,51,55,59,63,65,66,68,69,70,72,75,76,77,78,83,90,93,94,96,104,105,106,111,115,131,132,141,142,147,148,151,157");
		getConfig().addDefault("teleBlock",345);
		getConfig().options().copyDefaults(true);
		
        String[] ignoreBlocksString = getConfig().getString("ignoreBlocks","0,6,8,9,10,11,27,28,37,38,39,40,50,51,55,59,63,65,66,68,69,70,72,75,76,77,78,83,90,93,94,96,104,105,106,111,115,131,132,141,142,147,148,151,157").split(",");
		
		for(String s: ignoreBlocksString) ignoreBlocks.add((byte)(Short.parseShort(s)));
		
		teleBlock = getConfig().getInt("teleBlock",345);
		
		loadPermissions();
		
		// Register our events
		getServer().getPluginManager().registerEvents(this, this);
		
		PluginDescriptionFile pdfFile = this.getDescription();
		System.out.println( pdfFile.getName() + " version " + pdfFile.getVersion() + " enabled" );
	}
	public void onDisable() {
		// NOTE: All registered events are automatically unregistered when a plugin is disabled
		System.out.println("Unloading "+this.getDescription().getName());
	}
	
	public void loadPermissions() {
		if(getServer().getPluginManager().isPluginEnabled("PermissionsEx")){
			permissions = PermissionsEx.getPermissionManager();
		} else {
			Logger.getLogger("Minecraft").warning("PermissionsEx plugin not found.");
			return;
		}
	}
	
	public static boolean hasPermissions(Player player, String node) {
		if (permissions != null) {
			return permissions.has(player, node);
		} else {
			return player.isOp();
		}
	}
	
	public Boolean isLocationValid(Location l) {
		Location l2 = l.clone();
		l2.setY(l2.getY()+1);
		
		int m = l.getBlock().getTypeId();
		int m2 = l2.getBlock().getTypeId();
		
		if(!ignoreBlocks.contains((byte)m)) {
			System.out.println("Cannot teleport to "+l.getX()+","+l.getY()+","+l.getZ()+" because it contains an invalid type: "+l.getBlock().getType());
		}
		if(!ignoreBlocks.contains((byte)m2)) {
			System.out.println("Cannot teleport to "+l2.getX()+","+l2.getY()+","+l2.getZ()+" because it contains an invalid type: "+l2.getBlock().getType());
		}
		
		return ignoreBlocks.contains((byte)m) && ignoreBlocks.contains((byte)m2);
	}
	
	@EventHandler(priority = EventPriority.HIGHEST)
	public void onPlayerInteract(PlayerInteractEvent event) {
		Player player = event.getPlayer();
		
		if(player.getItemInHand().getTypeId() == teleBlock && (event.getAction() == Action.RIGHT_CLICK_AIR || event.getAction() == Action.RIGHT_CLICK_BLOCK)) {
			if(!hasPermissions(player, "teleporter.teleport")) return;
			 
			Block b = null;
			
			Location from = player.getLocation();
			
			b = player.getTargetBlock(ignoreBlocks,400);
			
			if(b == null) {
				System.out.println("Unable to find block. WTF.");
				return;
			}
				
			event.setCancelled(true);
			
			Location l = new Location(b.getWorld(),b.getX(),b.getY(),b.getZ(),from.getYaw(),from.getPitch());
			
			l.setX(Math.floor(l.getX())+0.5);
			l.setY(Math.floor(l.getY()));
			l.setZ(Math.floor(l.getZ())+0.5);
			
			int height = b.getWorld().getMaxHeight();
			while(l.getY() < height) {
				if(isLocationValid(l)) break;
				l.setY(l.getY()+1);
			}
			if(l.getY() == height) {
				System.out.println("No free block found");
			}
			
			player.teleport(l);
		}
    }
}

